/***{{{*******************************************************************
 * This file is part of libco - object library for C                     *
 * Copyright (c) 2002, 2003                                              *
 *     Peter Rockai (yenar) <yenar@host.sk>                              *
 *                                                                       *
 * This library is free software; you can redistribute it and/or         *
 * modify it under the terms of the GNU Lesser General Public            *
 * License as published by the Free Software Foundation; either          *
 * version 2 of the License, or (at your option) any later version.      *
 *                                                                       *
 * This library is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 * Lesser General Public License for more details.                       *
 *                                                                       *
 * You should have received a copy of the GNU Lesser General Public      *
 * License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330,                         *
 * Boston, MA 02111-1307  USA                                            *
 *******************************************************************}}}***/
/* {{{ file description */
/**
 * @file co_o_default.h
 * @brief Implementation of "default" object flavour.
 **/
/* }}} */

#ifndef __CO_O_DEFAULT_H__
#define __CO_O_DEFAULT_H__

#include "co_o_small.h"
#include "co_type.h"
#include "co_exception.h"
#include "co_debug.h"

/* {{{ co event macros */
/// Raise event in object @a o.
#define CO_EV(o, ...) (co_ev (o, COE (__VA_ARGS__)))

/// Connect object @a a event (signal) to object @a b message (slot)
#define CO_CNT(a, e, b, m) (co_cnt ((a), (COE (e)), b, COM (m), false))
#define CO_CNT_AD(a, e, b, m) (co_cnt ((a), (COE (e)), b, COM (m), true))
/* }}} */

/* {{{ */
#define CO_HEAD_EX_EV \
	co_ev_listen listen; \
	int check_listen:1; \
	int ev_running:1;
/* }}} */
/* {{{ */
struct co_ev_listen
	/// Object connection record - event listener (internal).
{
	/// Message ID to send if activated.
	co_id_t msg;
	/// Event ID, if raised connection is activated.
	co_id_t ev;
	/// Object to send message to.
	co o;
	/// Next list member.
	struct co_ev_listen *n;
	int deleted:1;
	int ad:1;
};
CO_TYPEDEF (co_ev_listen);
/* }}} */
/* {{{ */
struct co_o_default
{
	CO_HEAD;
	CO_HEAD_EX_SMALL;
	CO_HEAD_EX_EV;
};
CO_TYPEDEF (co_o_default);
/* }}} */

/// Raise event from object.
/**
 * This function is responsible for raising events. When event is invoked,
 * the object (in fact this function) will iterate over list of registered
 * listeners and send them messages as given by registration (arguments
 * are taken from event invoke handler). Currently there is no way to
 * predefine arguments to pass to message at registration time.
 *
 * @param o - object that raises event
 * @param ev - event id to check for handlers
 * @param varargs - arguments to pass to handlers
 **/
void co_ev (co o, co_id_t ev, ...);

co_v co_o_default_event (co this, co self, co_id_t message, va_list ap);

/// Connect event @a ev of object @a listener to @a msg of @a emitor.
void co_cnt (co emitor, co_id_t ev, co listener, co_id_t msg, bool ad);

co_v co_o_default_destroy (co _o, co self, co_id_t message, va_list ap);
co_v co_o_default_delete (co _o, co self, co_id_t message, va_list ap);

co_v co_o_default_clone_min (co this, co self, co_id_t message, va_list ap);
co_v co_o_default_clone (co this, co self, co_id_t message, va_list ap);
co_v co_o_default_clone_deep (co this, co self, co_id_t message, va_list ap);

#endif /* __CO_O_DEFAULT_H__ */
